div#config {
  position: absolute;
  width: 480px;
  top: 5px;
  bottom: 5px;
  background: #efefef;
  padding: 10px 10px;
  z-index: 999;
}

div#config div#buttons {
  position: absolute;
  bottom: 10px;
  right: 20px;
}

div#config div.setting {
  margin-top: 0.5em;
  margin-left: 160px;
  display: none;
}

div#config div.setting.active {
  display: block;
}

div#config div.setting div.field {
  float: left;
  margin-bottom: 10px;
  margin-right: 8px;
}

div#config div.setting div.field.clear {
  clear: both;
}

div#config label, div#config input {
  display: block;
  font-size: 0.8em;
}

div#config span {
  font-size: 0.8em;
}

div#config select {
  width: 135px;
  height: 100px;
  font-size: 0.8em;
    outline: 0 none;
}

div#config div#server_controls {
  position: absolute;
  bottom: 18px;
  left: 6px;
}

div#config div.controls {
  clear: both;
  font-size: 0.7em;
}

div#config div.controls a {
  text-decoration: none;
  margin: 0 5px 0 0;
}

div#config div.controls a:hover {
  text-shadow: none;
  text-decoration: underline;
}

div#config label {
  color: #666;
}

div#config ul#connections {
  list-style: none;
  position: absolute;
  top: 5px;
  bottom: 32px;
  left: 5px;
  margin: 0;
  padding: 0px;
  width: 150px;
  background: #fff;
  border-right: 1px solid #ccc;
  border-bottom: 1px solid #ccc;
}

div#config ul#connections li.header {
  background: #efefef;
  text-transform: uppercase;
  color: #777;
  letter-spacing: 1px;
  font-size: 0.7em;
  cursor: default;
}

div#config ul#connections li.header:hover {
  background: #efefef;
}

div#config ul#connections li {
  padding: 5px 8px;
  font-size: 0.9em;
  cursor: pointer;
    border: 1px solid #fff;
    border-bottom: none;
}

div#config ul#connections li:hover {
  background: #ffffcc;
}

div#config ul#connections li.disconnected.active,
div#config ul#connections li.connected.active {
  background: #3875d7;
  color: #fff;
}

div#config ul#connections li.disconnected {
  color: #999;
}

div#config ul#connections li.connected {
  color: #222;
}

body.config {
  background: #efefef;
}